import json, requests
from slugify import slugify
from bs4 import BeautifulSoup

try:
    from utils import login
except:
    from _8a_scraper.utils import login

def get_user_info(user):
    user = slugify(user)
    base_url = 'https://www.8a.nu/user/{}'
    user_url = base_url.format(user)
    r = requests.get(user_url)
    user_info = {'location': None, 'age': None, 'website': None,
                    'sponsors': None, 'started_climbing': None,
                    'occupation': None, 'other_interests': None,
                    'best_climbing_area': None, 'known_areas': None}

    if r.status_code==200:
        soup = BeautifulSoup(r.content, 'html.parser')
        top_div = soup.find('div', {'class': 'c-header-info'})
        sub_titles = top_div.find_all('p', {'class': 'sub-title'})
        for sub_title in sub_titles:
            if 'years old' in sub_title.text:
                age = int(sub_title.text.strip().replace('years old', ''))
                user_info['age'] = age
            else:
                user_info['location'] = sub_title.text.strip().replace(u'\xa0','')
        bottom_div = soup.find('div', {'class': 'user-info-body'})
        
        website_div = bottom_div.find('i', {'class': 'vl-website'})
        if website_div and len(website_div.text)>0:
            user_info['website'] = website_div.text.strip()
            
        for sponsor_div in bottom_div.find_all('div', {'class': 'sponsor-item'}):
            if not user_info['sponsors']:
                user_info['sponsors'] = []
            user_info['sponsors'].append(sponsor_div.text.strip())
        
        right_info = bottom_div.find_all('div', {'class': 'user-info-cell__right'})
        
        for i, div in enumerate(right_info):
            cell = div.find_all('div', {'class': 'cell'})[1]
            if i==0:
                user_info['started_climbing'] = cell.text.strip()
            elif i==1:
                user_info['occupation'] = cell.text.strip()
            elif i==2:
                user_info['other_interests'] = cell.text.strip()
            elif i==3:
                user_info['best_climbing_area'] = cell.text.strip()
            elif i==4:
                user_info['known_areas'] = cell.text.strip()
        
    return user_info


def get_recommended_ascents(user):
    user = slugify(user)
    driver = login()
    base_url = 'https://www.8a.nu/api/users/{}/recommended?pageSize=15&pageIndex={}'
    page_index = 0
    recommendations = []
    while True:
        url = base_url.format(user, page_index)
        driver.get(url)
        pre = driver.find_element_by_tag_name('pre').text
        data = json.loads(pre)
        if len(recommendations)<data['totalItems']:
            recommendations+=data['ascents']
            page_index+=1
        else:
            break
    driver.quit()
    return recommendations

def get_user_ascents(user, category):
    user = slugify(user)
    driver = login()
    base_url = 'https://www.8a.nu/api/users/{}/ascents?category={}&pageIndex={}&pageSize=50&sortfield=grade_desc&timeFilter=0&gradeFilter=0&typeFilter=&isAscented=true'
    ascents = []
    page_index = 0
    while True:
        url = base_url.format(user, category, page_index)
        driver.get(url)
        pre = driver.find_element_by_tag_name('pre').text
        data = json.loads(pre)
        if len(data['ascents'])==0:
            break
        else:
            ascents+=data['ascents']
            page_index+=1
    driver.quit()
    return ascents  



import unittest
from _8a_scraper.ascents import get_ascents

class TestAscents(unittest.TestCase):
    def test_get_ascents(self):
        climbers = get_ascents('Midnight Lightning', 'bouldering')
        expectedKeys = ['userAvatar', 'userName', 'userSlug', 'date', 'difficulty', 'isHard', 'isEasy', 'type', 'notes', 'rating', 'userPrivate', 'firstAscent', 'secondGo'] 
        self.assertListEqual(list(climbers[0].keys()), expectedKeys)

if __name__ == '__main__':
    unittest.main()



import unittest
from _8a_scraper.users import get_user_info, get_recommended_ascents, get_user_ascents

class TestUsers(unittest.TestCase):
    def test_get_user_info(self):
        user = 'Adam Ondra'
        user_info = get_user_info(user)
        self.assertEqual(user_info['location'], 'Brno, Czech Republic')

    def test_get_recommended_ascents(self):
        user = 'Adam Ondra'
        recs = get_recommended_ascents(user)
        self.assertGreater(len(recs), 0)

    def test_get_user_ascents(self):
        user = 'Adam Ondra'
        ascents = get_user_ascents(user, 'sportclimbing')
        self.assertGreater(len(ascents), 0)

        ascents = get_user_ascents(user, 'bouldering')
        self.assertGreater(len(ascents), 0)

if __name__ == '__main__':
    unittest.main()



import json
from bs4 import BeautifulSoup

try:
    from utils import login
except:
    from _8a_scraper.utils import login

def get_ascents(name, category):
    driver = login()
    name = name.lower().replace(' ', '+')
    url = f'https://www.8a.nu/api/search?query={name}&pageSize=100'
    driver.get(url)
    pre = driver.find_element_by_tag_name('pre').text
    items = json.loads(pre)['items']
    _category = 1 if category=='bouldering' else 0
    possibilities = [i for i in items if 'zlaggableName' in i and
            i['category']==_category]
    ascent = max(possibilities, key=lambda x: x['totalAscents'])
     
    base_url = 'https://www.8a.nu/api/crags/{}/{}/{}/sectors/{}/routes/{}/ascents?pageIndex={}&sortfield='
    page_index = 0
    ascents = []
    while True:
        url = base_url.format(category, ascent['countrySlug'], ascent['cragSlug'], ascent['sectorSlug'], ascent['zlaggableSlug'], page_index)
        driver.get(url)
        pre = driver.find_element_by_tag_name('pre').text
        data = json.loads(pre)
        if 'items' in data:
            ascents+=data['items']
        if 'pagination' in data and data['pagination']['hasNext']:
            page_index+=1
        else:
            break
    driver.quit()
    return ascents  



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="8a_scraper",
    version="0.0.4",
    author="Vishaal Agartha",
    author_email="vishaalagartha@gmail.com",
    license="MIT",
    description="A Python client for scraping data from 8a.nu",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/vishaalagartha/8a_scraper",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
    install_requires=[
        'beautifulsoup4==4.9.1',
        'bs4==0.0.1',
        'certifi==2020.6.20',
        'chardet==3.0.4',
        'idna==2.10',
        'python-slugify==4.0.1',
        'requests==2.24.0',
        'selenium==3.141.0',
        'soupsieve==2.0.1',
        'urllib3==1.25.9',
        'webdriver-manager==3.2.2'
    ],
    extras_require={
        'test': ['unittest'],
    },
    keywords=[
        "climbing",
        "rockclimbing",
        "bouldering",
        "sportclimbing",
        "8a",
        "8a.nu",
        "data mining",
        ]
)



import os
from selenium import webdriver
from webdriver_manager.chrome import ChromeDriverManager

def login():
    chrome_options = webdriver.ChromeOptions()
    chrome_options.add_argument("--headless")
    chrome_options.add_argument("--disable-dev-shm-usage")
    chrome_options.add_argument("--no-sandbox")
    chrome_options.add_argument(
        '--disable-blink-features=AutomationControlled')
    chrome_options.add_argument("window-size=1280,800")
    driver = webdriver.Chrome(
        executable_path=ChromeDriverManager().install(), options=chrome_options)
    driver.get('https://www.8a.nu/')
    driver.find_element_by_link_text('Log in').click()
    driver.find_element_by_id('username').send_keys(os.getenv('_8A_USERNAME'))
    driver.find_element_by_id('password').send_keys(os.getenv('_8A_PASSWORD'))
    driver.find_element_by_id('kc-login').submit()
    return driver
